/*
 * Ensys srl
 * 
 * Questo esempio recupera i dati raw dal sensore di VOC installato nella IndRsp
 * 
 * Verificare di aver abilitato la porta I2C0, ad esempio integrando nel file
 * 	/boot/config.txt 
 * le seguenti righe:
 * 	#i2c0 - periferiche
 * 	dtparam=i2c0=on
 * 	dtoverlay=i2c0-bcm2708,pins_28_29
 * 
 * Per compilare questo file utilizzare il comando:
 * 	gcc -o test ccs811b.c
 * Poi eseguire:
 * 	./test
 */
 
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#define HD__DEV__ADDR       (0x5bu)
#define HD__ID__ADDR        (0x20u)
#define HD__ID__VAL         (0x81u)

int32_t i2c__byte_read_byte(int32_t fd, uint8_t add, uint8_t *pdata){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,pdata,1u)==1u){
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_read_word(int32_t fd, uint8_t add, uint16_t *pdata){
        uint8_t buffer[2u];
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,buffer,2u)==2u){
                        *pdata = (buffer[0]<<8) | (buffer[1]);
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_read_buffer(int32_t fd, uint8_t add, uint8_t buffer[], uint8_t dim_buffer){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
		if( (buffer!=NULL) && (dim_buffer>0) ){
			if(read(fd,buffer,dim_buffer)==dim_buffer){
				ritorno = 0;
			}
		}
        }
        return ritorno;
}
int32_t i2c__byte_write_byte(int32_t fd, uint8_t add, uint8_t data){
        uint8_t buffer[2];
        int32_t ritorno = -1;
        buffer[0]=add;
        buffer[1]=data;
        if(write(fd,buffer,2u)==2u){
                ritorno = 0;
        }
        return ritorno;
}
int32_t i2c__byte_write_buffer(int32_t fd, uint8_t add, uint8_t buffer[], uint8_t dim_buffer){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
		if( (buffer!=NULL) && (dim_buffer>0) ){
			if(write(fd,buffer,dim_buffer)==dim_buffer){
				ritorno = 0;
			}
		}
        }
        return ritorno;
}

int32_t main(int32_t argc, uint8_t *argv[])
{
	int32_t fd;
	uint8_t buffer[10];
	uint8_t data;      
 	uint8_t status;      
        
        
	fd = open("/sys/class/gpio/export", O_WRONLY|O_SYNC);
	write(fd, "27", sizeof("27"));
	close(fd);

	fd = open("/sys/class/gpio/gpio27/direction", O_WRONLY|O_SYNC);
	write(fd, "out", sizeof("out"));
	close(fd);

	fd = open("/sys/class/gpio/gpio27/value", O_WRONLY|O_SYNC);
	write(fd, "0", sizeof("0"));
	close(fd);
        
        
        /* Apertura device */
       	if((fd=open("/dev/i2c-0", O_RDWR))<0)           printf("Error open\n");
        if(ioctl(fd, I2C_SLAVE, HD__DEV__ADDR)<0)        printf("Error set addr\n");
        
	/* Reset */
	uint8_t buffer_reset[] = {0x11, 0xE5, 0x72, 0x8A};
        if(i2c__byte_write_buffer(fd, 0xFFu, buffer_reset, 4)==0){
                printf("Reset OK\n");
        }
	sleep(1);
	
        /* Lettura ID */
        if(i2c__byte_read_byte(fd, HD__ID__ADDR, &data)==0){
                if(data==HD__ID__VAL)      printf("OK, trovato CCS811B!\n");
        }

	/* Start app */
        if(i2c__byte_write_buffer(fd, 0xF4u, NULL, 0u)==0){
                printf("Boot OK\n");
        }
	sleep(1);

        /* Set data rate */
        if(i2c__byte_write_byte(fd, 0x01, 0x10)==0){
                printf("Set data rate OK\n");
        }

        for(int32_t i=0;i<10000;){
                if(i2c__byte_read_byte(fd, 0x00, &status)!=0){
                        printf("Errore\n");
                        break;
                }
                else{
			if(status&0x01){
				i2c__byte_read_byte(fd, 0xE0, &data);
				printf("Error: %d\n", data);
			}
			else if(status&0x08){
				if(i2c__byte_read_buffer(fd, 0x02u, buffer, 8)==0){
					for(uint8_t j=0;j<8;j++) printf("%d ", buffer[j]);
					printf("\n");
				}
				i++;
			}
                }
		sleep(1);               
        }
       
        /* Chiusura device */
        close(fd);

	return (0);
}
